@@ -88,66 +88,6 @@ g_daemon_file_init (GDaemonFile *daemon_file)
 {
 }
 
-static char *
-canonicalize_path (const char *path)
-{
-  char *canon, *start, *p, *q;
-
-  if (*path != '/')
-    canon = g_strconcat ("/", path, NULL);
-  else
-    canon = g_strdup (path);
-
-  /* Skip initial slash */
-  start = canon + 1;
-
-  p = start;
-  while (*p != 0)
-    {
-      if (p[0] == '.' && (p[1] == 0 || p[1] == '/'))
-	{
-	  memmove (p, p+1, strlen (p+1)+1);
-	}
-      else if (p[0] == '.' && p[1] == '.' && (p[2] == 0 || p[2] == '/'))
-	{
-	  q = p + 2;
-	  /* Skip previous separator */
-	  p = p - 2;
-	  if (p < start)
-	    p = start;
-	  while (p > start && *p != '/')
-	    p--;
-	  if (*p == '/')
-	    p++;
-	  memmove (p, q, strlen (q)+1);
-	}
-      else
-	{
-	  /* Skip until next separator */
-	  while (*p != 0 && *p != '/')
-	    p++;
-
-	  /* Keep one separator */
-	  if (*p != 0)
-	    p++;
-	}
-
-      /* Remove additional separators */
-      q = p;
-      while (*q && *q == '/')
-	q++;
-
-      if (p != q)
-	memmove (p, q, strlen (q)+1);
-    }
-
-  /* Remove trailing slashes */
-  if (p > start && *(p-1) == '/')
-    *(p-1) = 0;
-  
-  return canon;
-}
-
 GFile *
 g_daemon_file_new (GMountSpec *mount_spec,
 		   const char *path)
@@ -156,7 +96,7 @@ g_daemon_file_new (GMountSpec *mount_spec,
 
   daemon_file = g_object_new (G_TYPE_DAEMON_FILE, NULL);
   daemon_file->mount_spec = g_mount_spec_get_unique_for (mount_spec);
-  daemon_file->path = canonicalize_path (path);
+  daemon_file->path = g_mount_spec_canonicalize_path (path);
  
   return G_FILE (daemon_file);
 }
@@ -261,12 +201,29 @@ g_daemon_file_get_parse_name (GFile *file)
 					      TRUE);
 }
 
+static GFile *
+new_file_for_new_path (GDaemonFile *daemon_file,
+		       const char *new_path)
+{
+  GFile *new_file;
+  GMountSpec *new_spec;
+
+  new_spec = _g_daemon_vfs_get_mount_spec_for_path (daemon_file->mount_spec,
+						    daemon_file->path,
+						    new_path);
+
+  new_file = g_daemon_file_new (new_spec, new_path);
+  g_mount_spec_unref (new_spec);
+
+  return new_file;
+}
+
 static GFile *
 g_daemon_file_get_parent (GFile *file)
 {
   GDaemonFile *daemon_file = G_DAEMON_FILE (file);
   const char *path;
-  GDaemonFile *parent;
+  GFile *parent;
   const char *base;
   char *parent_path;
   gsize len;    
@@ -286,11 +243,10 @@ g_daemon_file_get_parent (GFile *file)
   g_memmove (parent_path, path, len);
   parent_path[len] = 0;
 
-  parent = g_object_new (G_TYPE_DAEMON_FILE, NULL);
-  parent->mount_spec = g_mount_spec_ref (daemon_file->mount_spec);
-  parent->path = parent_path;
+  parent = new_file_for_new_path (daemon_file, parent_path);
+  g_free (parent_path);
   
-  return G_FILE (parent);
+  return parent;
 }
 
 static GFile *
@@ -387,10 +343,10 @@ g_daemon_file_resolve_relative_path (GFile *file,
   GFile *child;
 
   if (*relative_path == '/')
-    return g_daemon_file_new (daemon_file->mount_spec, relative_path);
-  
+    return new_file_for_new_path (daemon_file, relative_path);
+
   path = g_build_path ("/", daemon_file->path, relative_path, NULL);
-  child = g_daemon_file_new (daemon_file->mount_spec, path);
+  child = new_file_for_new_path (daemon_file, path);
   g_free (path);
   
   return child;
@@ -505,7 +461,7 @@ do_sync_2_path_call (GFile *file1,
 	  g_mount_info_unref (mount_info1);
 	  /* For copy this will cause the fallback code to be involved */
 	  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		       _("Operation not supported, files on different mounts"));
+		       "%s", _("Operation not supported, files on different mounts"));
 	  return NULL;
 	}
     }
@@ -790,7 +746,7 @@ g_daemon_file_query_info (GFile                *file,
       (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRUCT))
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from get_info"));
+		   "%s", _("Invalid return value from get_info"));
       goto out;
     }
 
@@ -993,7 +949,7 @@ g_daemon_file_read (GFile *file,
     {
       dbus_message_unref (reply);
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from open"));
+		   "%s", _("Invalid return value from open"));
       return NULL;
     }
   
@@ -1003,7 +959,7 @@ g_daemon_file_read (GFile *file,
   if (fd == -1)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Didn't get stream file descriptor"));
+		   "%s", _("Didn't get stream file descriptor"));
       return NULL;
     }
   
@@ -1052,7 +1008,7 @@ g_daemon_file_append_to (GFile *file,
     {
       dbus_message_unref (reply);
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from open"));
+		   "%s", _("Invalid return value from open"));
       return NULL;
     }
   
@@ -1062,7 +1018,7 @@ g_daemon_file_append_to (GFile *file,
   if (fd == -1)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Didn't get stream file descriptor"));
+		   "%s", _("Didn't get stream file descriptor"));
       return NULL;
     }
   
@@ -1111,7 +1067,7 @@ g_daemon_file_create (GFile *file,
     {
       dbus_message_unref (reply);
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from open"));
+		   "%s", _("Invalid return value from open"));
       return NULL;
     }
   
@@ -1121,7 +1077,7 @@ g_daemon_file_create (GFile *file,
   if (fd == -1)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Didn't get stream file descriptor"));
+		   "%s", _("Didn't get stream file descriptor"));
       return NULL;
     }
   
@@ -1173,7 +1129,7 @@ g_daemon_file_replace (GFile *file,
     {
       dbus_message_unref (reply);
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from open"));
+		   "%s", _("Invalid return value from open"));
       return NULL;
     }
   
@@ -1183,7 +1139,7 @@ g_daemon_file_replace (GFile *file,
   if (fd == -1)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Didn't get stream file descriptor"));
+		   "%s", _("Didn't get stream file descriptor"));
       return NULL;
     }
   
@@ -1267,6 +1223,7 @@ mount_mountable_async_cb (DBusMessage *reply,
   if (must_mount_location)
     {
       g_file_mount_enclosing_volume (file,
+				     0,
 				     mount_operation,
 				     cancellable,
 				     mount_mountable_location_mounted_cb,
@@ -1279,6 +1236,7 @@ mount_mountable_async_cb (DBusMessage *reply,
 
 static void
 g_daemon_file_mount_mountable (GFile               *file,
+			       GMountMountFlags     flags,
 			       GMountOperation     *mount_operation,
 			       GCancellable        *cancellable,
 			       GAsyncReadyCallback  callback,
@@ -1408,6 +1366,7 @@ typedef struct {
 } MountData;
 
 static void g_daemon_file_mount_enclosing_volume (GFile *location,
+						  GMountMountFlags flags,
 						  GMountOperation *mount_operation,
 						  GCancellable *cancellable,
 						  GAsyncReadyCallback callback,
@@ -1446,6 +1405,7 @@ mount_reply (DBusMessage *reply,
 
 static void
 g_daemon_file_mount_enclosing_volume (GFile *location,
+				      GMountMountFlags  flags,
 				      GMountOperation *mount_operation,
 				      GCancellable *cancellable,
 				      GAsyncReadyCallback callback,
@@ -1527,7 +1487,7 @@ g_daemon_file_query_filesystem_info (GFile                *file,
       (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRUCT))
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from get_filesystem_info"));
+		   "%s", _("Invalid return value from get_filesystem_info"));
       goto out;
     }
 
@@ -1566,7 +1526,7 @@ g_daemon_file_find_enclosing_mount (GFile *file,
 	       G_IO_ERROR_NOT_FOUND,
   /* translators: this is an error message when there is no user visible "mount" object
      corresponding to a particular path/uri */
-	       _("Could not find enclosing mount"));
+	       "%s", _("Could not find enclosing mount"));
   return NULL;
 }
 
@@ -1638,11 +1598,11 @@ g_daemon_file_set_display_name (GFile *file,
 				      0))
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from query_filesystem_info"));
+		   "%s", _("Invalid return value from query_filesystem_info"));
       goto out;
     }
 
-  file = g_daemon_file_new (daemon_file->mount_spec, new_path);
+  file = new_file_for_new_path (daemon_file, new_path);
   g_free (new_path);
 
  out:
@@ -1878,7 +1838,7 @@ g_daemon_file_copy (GFile                  *source,
   if (!G_IS_DAEMON_FILE (destination))
     {
       /* Fall back to default move */
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "Move not supported");
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "%s", "Move not supported");
       return FALSE;
     }
   daemon_dest = G_DAEMON_FILE (destination);
@@ -1897,7 +1857,7 @@ g_daemon_file_copy (GFile                  *source,
 	{
 	  /* This will cause the fallback code to be involved */
 	  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		       _("Operation not supported, files on different mounts"));
+		       "%s", _("Operation not supported, files on different mounts"));
 	  return FALSE;
 	}
     }
@@ -1970,7 +1930,7 @@ g_daemon_file_move (GFile                  *source,
       !G_IS_DAEMON_FILE (destination))
     {
       /* Fall back to default move */
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "Move not supported");
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "%s", "Move not supported");
       return FALSE;
     }
   
@@ -2046,7 +2006,7 @@ g_daemon_file_monitor_dir (GFile* file,
       g_mount_info_unref (mount_info);
       dbus_message_unref (reply);
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from monitor_dir"));
+		   "%s", _("Invalid return value from monitor_dir"));
       return NULL;
     }
   
@@ -2095,7 +2055,7 @@ g_daemon_file_monitor_file (GFile* file,
       g_mount_info_unref (mount_info);
       dbus_message_unref (reply);
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Invalid return value from monitor_file"));
+		   "%s", _("Invalid return value from monitor_file"));
       return NULL;
     }
   
