@@ -242,6 +242,7 @@ g_daemon_file_output_stream_init (GDaemonFileOutputStream *info)
 {
   info->output_buffer = g_string_new ("");
   info->input_buffer = g_string_new ("");
+  info->seq_nr = 1;
 }
 
 GFileOutputStream *
@@ -331,6 +332,7 @@ decode_error (GVfsDaemonSocketProtocolReply *reply, char *data, GError **error)
   g_set_error (error,
 	       g_quark_from_string (data),
 	       reply->arg1,
+               "%s",
 	       data + strlen (data) + 1);
 }
 
@@ -454,7 +456,8 @@ iterate_write_state_machine (GDaemonFileOutputStream *file, IOOperationData *io_
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
-			   _("Operation was cancelled"));
+			   "%s",
+                           _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
 	  
@@ -632,6 +635,7 @@ iterate_close_state_machine (GDaemonFileOutputStream *file, IOOperationData *io_
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
+                           "%s",
 			   _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
@@ -805,7 +809,7 @@ iterate_seek_state_machine (GDaemonFileOutputStream *file, IOOperationData *io_o
 	case SEEK_STATE_INIT:
 	  request = G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_SEEK_SET;
 	  if (op->seek_type == G_SEEK_CUR)
-	    request = G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_SEEK_CUR;
+	    op->offset = file->current_offset + op->offset;
 	  else if (op->seek_type == G_SEEK_END)
 	    request = G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_SEEK_END;
 	  append_request (file, request,
@@ -827,6 +831,7 @@ iterate_seek_state_machine (GDaemonFileOutputStream *file, IOOperationData *io_o
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
+                           "%s",
 			   _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
@@ -938,7 +943,7 @@ g_daemon_file_output_stream_seek (GFileOutputStream *stream,
   if (!file->can_seek)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		   _("Seek not supported on stream"));
+		   "%s", _("Seek not supported on stream"));
       return FALSE;
     }
   
