@@ -64,6 +64,7 @@ typedef struct _GVfsJobDelete          GVfsJobDelete;
 typedef struct _GVfsJobMakeDirectory   GVfsJobMakeDirectory;
 typedef struct _GVfsJobMakeSymlink     GVfsJobMakeSymlink;
 typedef struct _GVfsJobCopy            GVfsJobCopy;
+typedef struct _GVfsJobUpload          GVfsJobUpload;
 typedef struct _GVfsJobMove            GVfsJobMove;
 typedef struct _GVfsJobSetAttribute    GVfsJobSetAttribute;
 typedef struct _GVfsJobQueryAttributes GVfsJobQueryAttributes;
@@ -116,16 +117,20 @@ struct _GVfsBackendClass
 				 GMountSource *mount_source);
   void     (*unmount_mountable) (GVfsBackend *backend,
 				 GVfsJobUnmountMountable *job,
-				 const char *filename);
+				 const char *filename,
+				 GMountUnmountFlags flags);
   gboolean (*try_unmount_mountable)(GVfsBackend *backend,
-				 GVfsJobUnmountMountable *job,
-				 const char *filename);
+				    GVfsJobUnmountMountable *job,
+				    const char *filename,
+				    GMountUnmountFlags flags);
   void     (*eject_mountable)   (GVfsBackend *backend,
 				 GVfsJobUnmountMountable *job,
-				 const char *filename);
+				 const char *filename,
+				 GMountUnmountFlags flags);
   gboolean (*try_eject_mountable)(GVfsBackend *backend,
-				 GVfsJobUnmountMountable *job,
-				 const char *filename);
+				  GVfsJobUnmountMountable *job,
+				  const char *filename,
+				  GMountUnmountFlags flags);
   void     (*open_for_read)     (GVfsBackend *backend,
 				 GVfsJobOpenForRead *job,
 				 const char *filename);
@@ -292,6 +297,20 @@ struct _GVfsBackendClass
 				 GFileCopyFlags flags,
 				 GFileProgressCallback progress_callback,
 				 gpointer progress_callback_data);
+  void     (*upload)            (GVfsBackend *backend,
+				 GVfsJobUpload *job,
+				 const char *destination,
+				 const char *local_path,
+				 GFileCopyFlags flags,
+				 GFileProgressCallback progress_callback,
+				 gpointer progress_callback_data);
+  gboolean (*try_upload)        (GVfsBackend *backend,
+				 GVfsJobUpload *job,
+				 const char *destination,
+				 const char *local_path,
+				 GFileCopyFlags flags,
+				 GFileProgressCallback progress_callback,
+				 gpointer progress_callback_data);
   void     (*move)              (GVfsBackend *backend,
 				 GVfsJobMove *job,
 				 const char *source,
@@ -376,7 +395,7 @@ void        g_vfs_backend_unregister_mount               (GVfsBackend        *ba
 							  gpointer            user_data);
 const char *g_vfs_backend_get_backend_type               (GVfsBackend        *backend);
 const char *g_vfs_backend_get_display_name               (GVfsBackend        *backend);
-const char *g_vfs_backend_get_stable_name                  (GVfsBackend        *backend);
+const char *g_vfs_backend_get_stable_name                (GVfsBackend        *backend);
 const char *g_vfs_backend_get_icon_name                  (GVfsBackend        *backend);
 GMountSpec *g_vfs_backend_get_mount_spec                 (GVfsBackend        *backend);
 GVfsDaemon *g_vfs_backend_get_daemon                     (GVfsBackend        *backend);
