@@ -343,6 +343,7 @@ g_vfs_backend_burn_init (GVfsBackendBurn *burn_backend)
   GVfsBackend *backend = G_VFS_BACKEND (burn_backend);
   GMountSpec *mount_spec;
   
+  /* Translators: This is the name of the backend */
   g_vfs_backend_set_display_name (backend, _("Burn"));
   g_vfs_backend_set_icon_name (backend, "gnome-fs-client");
   g_vfs_backend_set_user_visible (backend, FALSE);
@@ -468,7 +469,7 @@ try_open_for_read (GVfsBackend *backend,
   
   if (stream)
     {
-      g_vfs_job_open_for_read_set_can_seek (job, g_file_input_stream_can_seek (stream));
+      g_vfs_job_open_for_read_set_can_seek (job, g_seekable_can_seek (G_SEEKABLE (stream)));
       g_vfs_job_open_for_read_set_handle (job, stream);
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
@@ -520,10 +521,10 @@ do_seek_on_read (GVfsBackend *backend,
   GFileInputStream *stream = _handle;
   
   error = NULL;
-  if (g_file_input_stream_seek (stream, offset, type,
-                                G_VFS_JOB (job)->cancellable, &error))
+  if (g_seekable_seek (G_SEEKABLE (stream), offset, type,
+                       G_VFS_JOB (job)->cancellable, &error))
     {
-      g_vfs_job_seek_read_set_offset (job, g_file_input_stream_tell(stream));
+      g_vfs_job_seek_read_set_offset (job, g_seekable_tell (G_SEEKABLE (stream)));
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
   else
@@ -647,7 +648,8 @@ file_info_from_node (VirtualNode *node,
   else
     {
       g_file_info_set_name (info, "/");
-      g_file_info_set_display_name (info, _("CD Burner"));
+      /* Translators: this is the display name of the backend */
+      g_file_info_set_display_name (info, _("CD/DVD Creator"));
     }
 }
 
@@ -822,10 +824,12 @@ try_upload (GVfsBackend *backend,
   
   if (g_stat (local_path, &stat_buf) == -1)
     {
+      int errsv = errno;
+
       g_vfs_job_failed (G_VFS_JOB (job),
                         G_IO_ERROR,
-                        g_io_error_from_errno (errno),
-                        g_strerror (errno));
+                        g_io_error_from_errno (errsv),
+                        "%s", g_strerror (errsv));
       return TRUE;
     }
 
