@@ -68,7 +68,7 @@
 /* TODO:
  *  Bugs: 
  *   - unmounting ?
- *   - copy dialog still visible after try_copy() finished successfully
+ *   - copy dialog still visible after try_copy() finished successfully  --> temporarily disabled do_copy ()
  * 
  * 
  *  Conceptual:
@@ -87,7 +87,7 @@
  */
 
 
-static void     
+static gboolean     
 inject_error (GVfsBackend *backend,
 			  GVfsJob *job,
 			  GVfsJobType job_type)
@@ -99,9 +99,11 @@ inject_error (GVfsBackend *backend,
   {
 	  g_print ("(II) inject_error: BANG! injecting error... \n");
       g_vfs_job_failed (G_VFS_JOB (job),
-			G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+			G_IO_ERROR, G_IO_ERROR_FAILED,
 			"Injected error");
-  } else g_vfs_job_succeeded (job); 
+      return FALSE;
+  } else g_vfs_job_succeeded (job);
+  return TRUE;
 }
 
 
@@ -177,14 +179,16 @@ G_DEFINE_TYPE (GVfsBackendLocalTest, g_vfs_backend_localtest, G_VFS_TYPE_BACKEND
 static void
 g_vfs_backend_localtest_init (GVfsBackendLocalTest *backend)
 {
-	//  Nothing in there
+	const char *c;
+
+	/*  Nothing in there */
 	g_print ("(II) g_vfs_backend_localtest_init \n");
 
-	//  env var conversion
+	/*  env var conversion */
 	backend->errorneous = -1;
 	backend->inject_op_types = -1;
 	
-	const char *c = g_getenv("GVFS_ERRORNEOUS");
+	c = g_getenv("GVFS_ERRORNEOUS");
 	if (c) {
 		backend->errorneous = g_ascii_strtoll(c, NULL, 0);
 		g_print ("(II) g_vfs_backend_localtest_init: setting 'errorneous' to '%d' \n", backend->errorneous);
@@ -202,9 +206,10 @@ g_vfs_backend_localtest_init (GVfsBackendLocalTest *backend)
 static void
 g_vfs_backend_localtest_finalize (GObject *object)
 {
+	GVfsBackendLocalTest *backend;
+
 	g_print ("(II) g_vfs_backend_localtest_finalize \n");
 
-	GVfsBackendLocalTest *backend;
 	backend = G_VFS_BACKEND_LOCALTEST (object);
 
     if (backend->test)
@@ -250,9 +255,11 @@ do_mount (GVfsBackend *backend,
 static void
 do_unmount (GVfsBackend *backend, GVfsJobUnmount *job)
 {
+  GVfsBackendLocalTest *op_backend;
+
   g_print ("(II) try_umount \n");
 
-  GVfsBackendLocalTest *op_backend = G_VFS_BACKEND_LOCALTEST (backend);
+  op_backend = G_VFS_BACKEND_LOCALTEST (backend);
   g_mount_spec_unref (op_backend->mount_spec);
   inject_error (backend, G_VFS_JOB (job), GVFS_JOB_UNMOUNT);
 }
@@ -275,34 +282,41 @@ do_enumerate (GVfsBackend *backend,
                GFileAttributeMatcher *attribute_matcher,
                GFileQueryInfoFlags flags)
 {
-  g_print ("(II) try_enumerate (filename = %s) \n", filename);
-
   GFile *file;
   GFileInfo *info;
   GError *error;
   GFileEnumerator *enumerator;
+  gboolean res;
   
+  g_print ("(II) try_enumerate (filename = %s) \n", filename);
+
   file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  g_assert (file != NULL);
+  res = TRUE;
 
   error = NULL;
   enumerator = g_file_enumerate_children (file, "*", flags, G_VFS_JOB (job)->cancellable, &error);
   if (enumerator) {
 	  error = NULL;
-      while ((info = g_file_enumerator_next_file (enumerator, G_VFS_JOB (job)->cancellable, &error)) != NULL) {
+	  while ((info = g_file_enumerator_next_file (enumerator, G_VFS_JOB (job)->cancellable, &error)) != NULL) {
     	  g_print ("  (II) try_enumerate (filename = %s): file '%s' \n", filename, g_file_info_get_attribute_string(info, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME));
     	  g_vfs_job_enumerate_add_info (job, info);
       }
 	  if (error) {
 		  g_print ("  (EE) try_enumerate: error: %s \n", error->message);
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
+		  g_error_free (error);
+		  res = FALSE;
 	  } else 
-		  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_ENUMERATE);
+		  res = inject_error (backend, G_VFS_JOB (job), GVFS_JOB_ENUMERATE);
+	  
 	  error = NULL;
       g_file_enumerator_close (enumerator, G_VFS_JOB (job)->cancellable, &error);
 	  if (error) {
 		  g_print ("  (EE) try_enumerate: g_file_enumerator_close() error: %s \n", error->message);
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
+		  g_error_free (error);
+		  res = FALSE;
 	  } 
       g_object_unref (enumerator);
       g_print ("  (II) try_enumerate: success. \n");
@@ -310,15 +324,21 @@ do_enumerate (GVfsBackend *backend,
 	  if (error) {
 		  g_print ("  (EE) try_enumerate: error: %s \n", error->message);
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
+		  g_error_free (error);
+		  res = FALSE;
 	  } else { 
 		  g_print ("  (EE) try_enumerate: error == NULL \n");
 		  g_vfs_job_failed (G_VFS_JOB (job),
 					G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
 					"Error try_enumerate");
+		  res = FALSE;
 	  }
   }
+
   g_object_unref (file);
-  g_vfs_job_enumerate_done (job);  
+  if (res) {
+	  g_vfs_job_enumerate_done (job);  
+  }
   
   g_print ("(II) try_enumerate done. \n");
 }
@@ -332,10 +352,13 @@ do_query_info (GVfsBackend *backend,
                 GFileInfo *info,
                 GFileAttributeMatcher *matcher)
 {
+  GFile *file;
+  GFileInfo *info2;
+
   g_print ("(II) try_query_info (filename = %s) \n", filename);
   
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  GFileInfo *info2 = get_g_file_info_from_local (filename, file, "*", flags, G_VFS_JOB (job));
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  info2 = get_g_file_info_from_local (filename, file, "*", flags, G_VFS_JOB (job));
 
   if (info2) {
       g_file_info_copy_into (info2, info);
@@ -355,28 +378,31 @@ do_query_fs_info (GVfsBackend *backend,
 		  GFileInfo *info,
 		  GFileAttributeMatcher *attribute_matcher)
 {
-  g_print ("(II) try_query_fs_info (filename = %s) \n", filename);
-
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  GFile *file;
   GFileInfo *info2;
   GError *error;
 
+  g_print ("(II) try_query_fs_info (filename = %s) \n", filename);
+
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+
   if (file) {
 	  error = NULL;  
 	  info2 = g_file_query_filesystem_info (file, "fs:*", G_VFS_JOB (job)->cancellable, &error);
 	  if ((error) || (! info2) ) {
 		  g_print ("  (EE) try_query_fs_info (filename = '%s'): g_file_query_filesystem_info failed: %s \n", filename, error->message);
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
-	  } else
-	  {
+		  g_error_free (error);
+	  } else {
 	      g_file_info_copy_into (info2, info);
 	      g_object_unref (info2);
 	      g_object_unref (file);
 	      inject_error (backend, G_VFS_JOB (job), GVFS_JOB_QUERY_FS_INFO);
 	      g_print ("(II) try_query_fs_info success. \n");
 	  }
-  } else
+  } else {
 	  g_print ("(EE) try_query_fs_info failed. \n");
+  }
 }
 
 
@@ -385,12 +411,14 @@ do_query_settable_attributes (GVfsBackend *backend,
 					     GVfsJobQueryAttributes *job,
 					     const char *filename)
 {
-  g_print ("(II) try_query_settable_attributes (filename = '%s') \n", filename);
-
   GFileAttributeInfoList *attr_list;
   GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
+
+  g_print ("(II) try_query_settable_attributes (filename = '%s') \n", filename);
+
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -402,10 +430,12 @@ do_query_settable_attributes (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_query_settable_attributes: g_file_query_settable_attributes == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_query_settable_attributes: file == NULL \n");
+  }
 }
 
 static void
@@ -413,12 +443,14 @@ do_query_writable_namespaces (GVfsBackend *backend,
 					     GVfsJobQueryAttributes *job,
 					     const char *filename)
 {
-  g_print ("(II) try_query_writable_namespaces (filename = '%s') \n", filename);
-
   GFileAttributeInfoList *attr_list;
   GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
+
+  g_print ("(II) try_query_writable_namespaces (filename = '%s') \n", filename);
+
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -430,10 +462,12 @@ do_query_writable_namespaces (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_query_writable_namespaces: g_file_query_writable_namespaces == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_query_writable_namespaces: file == NULL \n");
+  }
 }
 
 
@@ -450,11 +484,13 @@ do_make_directory (GVfsBackend *backend,
                     GVfsJobMakeDirectory *job,
                     const char *filename)
 {
+  GError *error;
+  GFile *file;
+
   g_print ("(II) try_make_directory (filename = %s) \n", filename);
 
-  GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -464,10 +500,12 @@ do_make_directory (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_make_directory: g_file_make_directory == FALSE \n");
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_make_directory: file == NULL \n");
+  }
 }
 
 
@@ -476,11 +514,13 @@ do_delete (GVfsBackend *backend,
             GVfsJobDelete *job,
             const char *filename)
 {
+  GError *error;
+  GFile *file;
+
   g_print ("(II) try_delete (filename = %s) \n", filename);
 
-  GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -490,10 +530,12 @@ do_delete (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_delete: g_file_delete == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_delete: file == NULL \n");
+  }
 }
 
 
@@ -502,11 +544,13 @@ do_trash (GVfsBackend *backend,
 			GVfsJobTrash *job,
             const char *filename)
 {
+  GError *error;
+  GFile *file;
+
   g_print ("(II) try_trash (filename = %s) \n", filename);
 
-  GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -516,10 +560,12 @@ do_trash (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_trash: g_file_trash == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_trash: file == NULL \n");
+  }
 }
 
 
@@ -529,11 +575,13 @@ do_make_symlink (GVfsBackend *backend,
                   const char *filename,
                   const char *symlink_value)
 {
+  GError *error;
+  GFile *file;
+
   g_print ("(II) try_make_symlink ('%s' --> '%s') \n", filename, symlink_value);
 
-  GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -543,10 +591,12 @@ do_make_symlink (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_make_symlink: g_file_make_symbolic_link == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_make_symlink: file == NULL \n");
+  }
 }
 
 
@@ -559,14 +609,14 @@ do_copy (GVfsBackend *backend,
 		 GFileProgressCallback progress_callback,
 		 gpointer progress_callback_data)
 {
-  g_print ("(II) try_copy '%s' --> '%s' \n", source, destination);
-	  
   GFile *src_file, *dst_file;
   GError *error;
   
+  g_print ("(II) try_copy '%s' --> '%s' \n", source, destination);
+	  
   src_file = get_g_file_from_local (source, G_VFS_JOB (job));
   dst_file = get_g_file_from_local (destination, G_VFS_JOB (job));
-  g_assert(src_file != NULL);
+  g_assert (src_file != NULL);
 
   if (src_file) {
 	  error = NULL;
@@ -578,11 +628,13 @@ do_copy (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_copy: g_file_copy == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (src_file);
 	  g_object_unref (dst_file);
-  } else 
+  } else { 
 	  g_print ("  (EE) try_copy: file == NULL \n");
+  }
 }
 
 
@@ -595,14 +647,14 @@ do_move (GVfsBackend *backend,
           GFileProgressCallback progress_callback,
           gpointer progress_callback_data)
 {
-  g_print ("(II) try_move '%s' --> '%s' \n", source, destination);
-	  
   GFile *src_file, *dst_file;
   GError *error;
   
+  g_print ("(II) try_move '%s' --> '%s' \n", source, destination);
+	  
   src_file = get_g_file_from_local (source, G_VFS_JOB (job));
   dst_file = get_g_file_from_local (destination, G_VFS_JOB (job));
-  g_assert(src_file != NULL);
+  g_assert (src_file != NULL);
 
   if (src_file) {
 	  error = NULL;
@@ -614,11 +666,13 @@ do_move (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_move: g_file_move == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (src_file);
 	  g_object_unref (dst_file);
-  } else 
+  } else { 
 	  g_print ("  (EE) try_move: file == NULL \n");
+  }
 }
 
 
@@ -629,11 +683,13 @@ do_set_display_name (GVfsBackend *backend,
                       const char *filename,
                       const char *display_name)
 {
+  GError *error;
+  GFile *file;
+
   g_print ("(II) try_set_display_name '%s' --> '%s' \n", filename, display_name);
 
-  GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -654,10 +710,12 @@ do_set_display_name (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_set_display_name: g_file_set_display_name == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_set_display_name: file == NULL \n");
+  }
 }
 
 
@@ -670,11 +728,13 @@ do_set_attribute (GVfsBackend *backend,
 		 gpointer value_p,
 		 GFileQueryInfoFlags flags)
 {
+  GError *error;
+  GFile *file;
+
   g_print ("(II) try_set_attribute (filename = '%s', attribute = '%s') \n", filename, attribute);
 
-  GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (set_attribute));
-  g_assert(file != NULL);
+  file = get_g_file_from_local (filename, G_VFS_JOB (set_attribute));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
@@ -684,10 +744,12 @@ do_set_attribute (GVfsBackend *backend,
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (set_attribute), error); 
 		  g_print ("  (EE) try_set_attribute: g_file_set_attribute == FALSE, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_set_attribute: file == NULL \n");
+  }
 }
 
 
@@ -701,7 +763,7 @@ do_set_attribute (GVfsBackend *backend,
  * 
  */
 
-//  MonitorProxy has been stolen from gvfsbackendtrash
+/*  MonitorProxy has been stolen from gvfsbackendtrash  */
 typedef struct {
   GVfsMonitor *vfs_monitor;
   GObject *monitor;
@@ -755,18 +817,20 @@ create_dir_file_monitor (GVfsBackend *backend,
 				      GFileMonitorFlags flags,
 				      const gboolean is_dir_monitor)
 {
-  GVfsBackendLocalTest *op_backend = G_VFS_BACKEND_LOCALTEST (backend);
-
   GObject *monitor;
   MonitorProxy *proxy;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
   
-  if (is_dir_monitor) monitor = G_OBJECT(g_file_monitor_directory (file, flags, G_VFS_JOB (job)->cancellable, NULL));
-  else monitor = G_OBJECT(g_file_monitor_file (file, flags, G_VFS_JOB (job)->cancellable, NULL));
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
+  
+  if (is_dir_monitor) { 
+	  monitor = G_OBJECT (g_file_monitor_directory (file, flags, G_VFS_JOB (job)->cancellable, NULL));
+  } else {
+	  monitor = G_OBJECT (g_file_monitor_file (file, flags, G_VFS_JOB (job)->cancellable, NULL));
+  }
 	  
-  if (monitor)
-    {
+  if (monitor) {
       proxy = g_new0 (MonitorProxy, 1); 
       proxy->vfs_monitor = g_vfs_monitor_new (backend);
       proxy->monitor = monitor;
@@ -782,8 +846,7 @@ create_dir_file_monitor (GVfsBackend *backend,
       inject_error (backend, G_VFS_JOB (job), GVFS_JOB_CREATE_DIR_MONITOR);
       g_print ("(II) create_dir_file_monitor success. \n");
     }
-  else
-    {
+  else  {
       g_print ("  (EE) create_dir_file_monitor: monitor == NULL \n");
       g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                         G_IO_ERROR_NOT_SUPPORTED,
@@ -829,28 +892,32 @@ do_open_for_read (GVfsBackend *backend,
                    GVfsJobOpenForRead *job,
                    const char *filename)
 {
-  g_print ("(II) try_open_for_read (filename = '%s') \n", filename);
-  
   GFileInputStream *stream;
   GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
+
+  g_print ("(II) try_open_for_read (filename = '%s') \n", filename);
+  
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
 	  stream = g_file_read (file, G_VFS_JOB (job)->cancellable, &error);
 	  if (stream) {
-	      g_vfs_job_open_for_read_set_can_seek (job, g_file_input_stream_can_seek(stream));
-	      g_vfs_job_open_for_read_set_handle (job, stream);
+		  g_vfs_job_open_for_read_set_can_seek (job, g_seekable_can_seek (G_SEEKABLE (stream)));
+		  g_vfs_job_open_for_read_set_handle (job, stream);
 		  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_OPEN_FOR_READ);
 		  g_print ("(II) try_open_for_read success. \n");
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_open_for_read: stream == NULL, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_open_for_read: file == NULL \n");
+  }
 }
 
 
@@ -861,15 +928,18 @@ do_read (GVfsBackend *backend,
           char *buffer,
           gsize bytes_requested)
 {
+  GError *error;
+  GFileInputStream *stream = _handle;
+  gssize s;
+
   g_print ("(II) try_read (handle = '%lx', buffer = '%lx', bytes_requested = %ld) \n", 
 		  (long int)_handle, (long int)buffer, (long int)bytes_requested);
 
-  GError *error;
-  GFileInputStream *stream = _handle;
-  g_assert(stream != NULL);
+  g_assert (stream != NULL);
   
   error = NULL;
-  gssize s = g_input_stream_read (G_INPUT_STREAM(stream), buffer, bytes_requested, G_VFS_JOB (job)->cancellable, &error); 
+  s = g_input_stream_read (G_INPUT_STREAM (stream), buffer, bytes_requested, 
+		  				   G_VFS_JOB (job)->cancellable, &error); 
   if (s >= 0) {
       g_vfs_job_read_set_size (job, s);
 	  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_READ);
@@ -877,6 +947,7 @@ do_read (GVfsBackend *backend,
   } else  {
 	  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 	  g_print ("  (EE) try_read: g_input_stream_read() failed, error: %s \n", error->message);
+	  g_error_free (error);
   }
 }
 
@@ -888,20 +959,22 @@ do_seek_on_read (GVfsBackend *backend,
                   goffset    offset,
                   GSeekType  type)
 {
-  g_print ("(II) try_seek_on_read (handle = '%lx', offset = %ld) \n", (long int)_handle, (long int)offset);
-
   GError *error;
   GFileInputStream *stream = _handle;
-  g_assert(stream != NULL);
+
+  g_print ("(II) try_seek_on_read (handle = '%lx', offset = %ld) \n", (long int)_handle, (long int)offset);
+
+  g_assert (stream != NULL);
   
   error = NULL;
-  if (g_file_input_stream_seek (stream, offset, type, G_VFS_JOB (job)->cancellable, &error)) {
-      g_vfs_job_seek_read_set_offset (job, g_file_input_stream_tell(stream));
+  if (g_seekable_seek (G_SEEKABLE (stream), offset, type, G_VFS_JOB (job)->cancellable, &error)) {
+	  g_vfs_job_seek_read_set_offset (job, g_seekable_tell (G_SEEKABLE (stream)));
 	  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_SEEK_ON_READ);
 	  g_print ("(II) try_seek_on_read success. \n");
   } else  {
 	  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 	  g_print ("  (EE) try_seek_on_read: g_file_input_stream_seek() failed, error: %s \n", error->message);
+	  g_error_free (error);
   }
 }
 
@@ -911,20 +984,22 @@ do_close_read (GVfsBackend *backend,
                 GVfsJobCloseRead *job,
                 GVfsBackendHandle _handle)
 {
-  g_print ("(II) try_close_read (handle = '%lx') \n", (long int)_handle);
-
   GError *error;
   GFileInputStream *stream = _handle;
-  g_assert(stream != NULL);
+
+  g_print ("(II) try_close_read (handle = '%lx') \n", (long int)_handle);
+
+  g_assert (stream != NULL);
   
   error = NULL;
-  if (g_input_stream_close (G_INPUT_STREAM(stream), G_VFS_JOB (job)->cancellable, &error)) {
+  if (g_input_stream_close (G_INPUT_STREAM (stream), G_VFS_JOB (job)->cancellable, &error)) {
 	  g_object_unref (stream);
 	  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_CLOSE_READ);
 	  g_print ("(II) try_close_read success. \n");
   } else  {
 	  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 	  g_print ("  (EE) try_close_read: g_input_stream_close() failed, error: %s \n", error->message);
+	  g_error_free (error);
   }
 }
 
@@ -935,35 +1010,40 @@ do_append_to (GVfsBackend *backend,
                const char *filename,
                GFileCreateFlags flags)
 {
-  g_print ("(II) try_append_to (filename = %s) \n", filename);
-
   GFileOutputStream *stream;
   GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
+
+  g_print ("(II) try_append_to (filename = %s) \n", filename);
+
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
 	  stream = g_file_append_to (file, flags, G_VFS_JOB (job)->cancellable, &error);
 	  if (stream) {
-		  //  Should seek at the end of the file here
-		  if ((g_file_output_stream_seek (stream, 0, G_SEEK_END, G_VFS_JOB (job)->cancellable, &error)) && (! error))
-			  g_vfs_job_open_for_write_set_initial_offset (job, g_file_output_stream_tell(stream));
-		  else
-			  g_print ("  (EE) try_append_to: error during g_file_output_stream_seek(), error: %s \n", error->message);
+		  /*  Should seek at the end of the file here  */
+		  if ((g_seekable_seek (G_SEEKABLE (stream), 0, G_SEEK_END, G_VFS_JOB (job)->cancellable, &error)) && (! error)) {
+			  g_vfs_job_open_for_write_set_initial_offset (job, g_seekable_tell (G_SEEKABLE (stream)));
+		  } else {
+		  	  g_print ("  (EE) try_append_to: error during g_file_output_stream_seek(), error: %s \n", error->message);
+		  }
 
-	      g_vfs_job_open_for_write_set_can_seek (job, g_file_output_stream_can_seek(stream));
-	      g_vfs_job_open_for_write_set_handle (job, stream);
+		  g_vfs_job_open_for_write_set_can_seek (job, g_seekable_can_seek (G_SEEKABLE (stream)));
+		  g_vfs_job_open_for_write_set_handle (job, stream);
 		  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_APPEND_TO);
 
-	      g_print ("(II) try_append_to success. \n");
+		  g_print ("(II) try_append_to success. \n");
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_append_to: stream == NULL, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_append_to: file == NULL \n");
+  }
 }
 
 
@@ -973,28 +1053,32 @@ do_create (GVfsBackend *backend,
             const char *filename,
             GFileCreateFlags flags)
 {
-  g_print ("(II) try_create (filename = %s) \n", filename);
-
   GFileOutputStream *stream;
   GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
+
+  g_print ("(II) try_create (filename = %s) \n", filename);
+
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
 	  stream = g_file_create (file, flags, G_VFS_JOB (job)->cancellable, &error);
 	  if (stream) {
-	      g_vfs_job_open_for_write_set_can_seek (job, g_file_output_stream_can_seek(stream));
-	      g_vfs_job_open_for_write_set_handle (job, stream);
+		  g_vfs_job_open_for_write_set_can_seek (job, g_seekable_can_seek (G_SEEKABLE (stream)));
+		  g_vfs_job_open_for_write_set_handle (job, stream);
 		  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_CREATE);
 		  g_print ("(II) try_create success. \n");
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_create: stream == NULL, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_create: file == NULL \n");
+  }
 }
 
 
@@ -1006,28 +1090,32 @@ do_replace (GVfsBackend *backend,
              gboolean make_backup,
              GFileCreateFlags flags)
 {
-  g_print ("(II) try_replace (filename = '%s', etag = '%s') \n", filename, etag);
-
   GFileOutputStream *stream;
   GError *error;
-  GFile *file = get_g_file_from_local (filename, G_VFS_JOB (job));
-  g_assert(file != NULL);
+  GFile *file;
+
+  g_print ("(II) try_replace (filename = '%s', etag = '%s') \n", filename, etag);
+
+  file = get_g_file_from_local (filename, G_VFS_JOB (job));
+  g_assert (file != NULL);
 
   if (file) {
 	  error = NULL;
 	  stream = g_file_replace (file, etag, make_backup, flags, G_VFS_JOB (job)->cancellable, &error);
 	  if (stream) {
-	      g_vfs_job_open_for_write_set_can_seek (job, g_file_output_stream_can_seek(stream));
-	      g_vfs_job_open_for_write_set_handle (job, stream);
+		  g_vfs_job_open_for_write_set_can_seek (job, g_seekable_can_seek (G_SEEKABLE (stream)));
+		  g_vfs_job_open_for_write_set_handle (job, stream);
 		  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_REPLACE);
 		  g_print ("(II) try_replace success. \n");
 	  } else {
 		  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 		  g_print ("  (EE) try_replace: stream == NULL, error: %s \n", error->message);
+		  g_error_free (error);
 	  }
 	  g_object_unref (file);
-  } else
+  } else {
 	  g_print ("  (EE) try_replace: file == NULL \n");
+  }
 }
 
 
@@ -1038,15 +1126,17 @@ do_write (GVfsBackend *backend,
            char *buffer,
            gsize buffer_size)
 {
+  GError *error;
+  GFileOutputStream *stream = _handle;
+  gssize s;
+
   g_print ("(II) try_write (handle = '%lx', buffer = '%lx', buffer_size = %ld) \n", 
 		  (long int)_handle, (long int)buffer, (long int)buffer_size);
 
-  GError *error;
-  GFileOutputStream *stream = _handle;
-  g_assert(stream != NULL);
+  g_assert (stream != NULL);
   
   error = NULL;
-  gssize s = g_output_stream_write (G_OUTPUT_STREAM(stream), buffer, buffer_size, G_VFS_JOB (job)->cancellable, &error); 
+  s = g_output_stream_write (G_OUTPUT_STREAM (stream), buffer, buffer_size, G_VFS_JOB (job)->cancellable, &error); 
   if (s >= 0) {
 	  g_vfs_job_write_set_written_size (job, s);
 	  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_WRITE);
@@ -1054,6 +1144,7 @@ do_write (GVfsBackend *backend,
   } else  {
 	  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 	  g_print ("  (EE) try_write: g_output_stream_write() failed, error: %s \n", error->message);
+	  g_error_free (error);
   }
 }
 
@@ -1065,20 +1156,22 @@ do_seek_on_write (GVfsBackend *backend,
                    goffset    offset,
                    GSeekType  type)
 {
-  g_print ("(II) try_seek_on_write (handle = '%lx', offset = %ld) \n", (long int)_handle, (long int)offset);
-
   GError *error;
   GFileOutputStream *stream = _handle;
-  g_assert(stream != NULL);
+
+  g_print ("(II) try_seek_on_write (handle = '%lx', offset = %ld) \n", (long int)_handle, (long int)offset);
+
+  g_assert (stream != NULL);
   
   error = NULL;
-  if (g_file_output_stream_seek (stream, offset, type, G_VFS_JOB (job)->cancellable, &error)) {
-	  g_vfs_job_seek_write_set_offset (job, g_file_output_stream_tell(stream));
+  if (g_seekable_seek (G_SEEKABLE (stream), offset, type, G_VFS_JOB (job)->cancellable, &error)) {
+	  g_vfs_job_seek_write_set_offset (job, g_seekable_tell (G_SEEKABLE (stream)));
 	  inject_error (backend, G_VFS_JOB (job), GVFS_JOB_SEEK_ON_WRITE);
 	  g_print ("(II) try_seek_on_write success. \n");
   } else  {
 	  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 	  g_print ("  (EE) try_seek_on_write: g_file_output_stream_seek() failed, error: %s \n", error->message);
+	  g_error_free (error);
   }
 }
 
@@ -1088,11 +1181,12 @@ do_close_write (GVfsBackend *backend,
                  GVfsJobCloseWrite *job,
                  GVfsBackendHandle _handle)
 {
-  g_print ("(II) try_close_write (handle = '%lx') \n", (long int)_handle);
-
   GError *error;
   GFileOutputStream *stream = _handle;
-  g_assert(stream != NULL);
+
+  g_print ("(II) try_close_write (handle = '%lx') \n", (long int)_handle);
+
+  g_assert (stream != NULL);
   
   error = NULL;
   if (g_output_stream_close (G_OUTPUT_STREAM(stream), G_VFS_JOB (job)->cancellable, &error)) {
@@ -1102,6 +1196,7 @@ do_close_write (GVfsBackend *backend,
   } else  {
 	  g_vfs_job_failed_from_error (G_VFS_JOB (job), error); 
 	  g_print ("  (EE) try_close_write: g_input_stream_close() failed, error: %s \n", error->message);
+	  g_error_free (error);
   }
 }
 
@@ -1141,7 +1236,8 @@ g_vfs_backend_localtest_class_init (GVfsBackendLocalTestClass *klass)
   backend_class->replace = do_replace;
   backend_class->write = do_write;
   backend_class->seek_on_write = do_seek_on_write;
-  backend_class->copy = do_copy;
+/*  -- disabled, read/write operations can handle copy correctly  */  
+/*   backend_class->copy = do_copy; */ 
   backend_class->move = do_move;
   backend_class->make_symlink = do_make_symlink;
   backend_class->make_directory = do_make_directory;
