@@ -246,7 +246,7 @@ look_for_stderr_errors (GVfsBackend *backend, GError **error)
           /* Error (real or WOULDBLOCK) or EOF */
           g_set_error (error,
                        G_IO_ERROR, G_IO_ERROR_FAILED,
-                       _("ssh program unexpectedly exited"));
+                       "%s", _("ssh program unexpectedly exited"));
           return;
         }
       
@@ -254,35 +254,35 @@ look_for_stderr_errors (GVfsBackend *backend, GError **error)
         {
           g_set_error (error,
                        G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
-                       _("Permission denied"));
+                       "%s", _("Permission denied"));
           return;
         }
       else if (strstr (line, "Name or service not known") != NULL)
         {
           g_set_error (error,
                        G_IO_ERROR, G_IO_ERROR_HOST_NOT_FOUND,
-                       _("Hostname not known"));
+                       "%s", _("Hostname not known"));
           return;
         }
       else if (strstr (line, "No route to host") != NULL)
         {
           g_set_error (error,
                        G_IO_ERROR, G_IO_ERROR_HOST_NOT_FOUND,
-                       _("No route to host"));
+                       "%s", _("No route to host"));
           return;
         }
       else if (strstr (line, "Connection refused") != NULL)
         {
           g_set_error (error,
                        G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
-                       _("Connection refused by server"));
+                       "%s", _("Connection refused by server"));
           return;
         }
       else if (strstr (line, "Host key verification failed") != NULL) 
         {
           g_set_error (error,
                        G_IO_ERROR, G_IO_ERROR_FAILED,
-                       _("Host key verification failed"));
+                       "%s", _("Host key verification failed"));
           return;
         }
       
@@ -367,7 +367,7 @@ spawn_ssh (GVfsBackend *backend,
     {
       g_set_error (error,
 		   G_IO_ERROR, G_IO_ERROR_FAILED,
-		   _("Unable to spawn ssh program"));
+		   "%s", _("Unable to spawn ssh program"));
       return FALSE;
     }
 #else
@@ -486,7 +486,7 @@ wait_for_reply (GVfsBackend *backend, int stdout_fd, GError **error)
     {
       g_set_error (error,
 		   G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-		   _("Timed out when logging in"));
+		   "%s", _("Timed out when logging in"));
       return FALSE;
     }
   return TRUE;
@@ -643,7 +643,7 @@ handle_login (GVfsBackend *backend,
 	{
 	  g_set_error (error,
 		       G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-		       _("Timed out when logging in"));
+		       "%s", _("Timed out when logging in"));
 	  ret_val = FALSE;
 	  break;
 	}
@@ -705,7 +705,7 @@ handle_login (GVfsBackend *backend,
 	    {
 	      g_set_error (error,
 			   G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
-			   _("Password dialog cancelled"));
+			   "%s", _("Password dialog cancelled"));
 	      ret_val = FALSE;
 	      break;
 	    }
@@ -721,7 +721,7 @@ handle_login (GVfsBackend *backend,
 	    {
 	      g_set_error (error,
 			   G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
-			   _("Can't send password"));
+			   "%s", _("Can't send password"));
 	      ret_val = FALSE;
 	      break;
 	    }
@@ -1087,7 +1087,7 @@ do_mount (GVfsBackend *backend,
   
   if (g_data_input_stream_read_byte (reply, NULL, NULL) != SSH_FXP_VERSION)
     {
-      g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED, _("Protocol error"));
+      g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED, "%s", _("Protocol error"));
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
       g_error_free (error);
       return;
@@ -1110,7 +1110,7 @@ do_mount (GVfsBackend *backend,
 
   if (!get_uid_sync (op_backend))
     {
-      g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED, _("Protocol error"));
+      g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED, "%s", _("Protocol error"));
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
       g_error_free (error);
       return;
@@ -1302,6 +1302,11 @@ parse_attributes (GVfsBackendSftp *backend,
   if (basename != NULL && basename[0] == '.')
     g_file_info_set_is_hidden (info, TRUE);
 
+  if (basename != NULL)
+    g_file_info_set_name (info, basename);
+  else
+    g_file_info_set_name (info, "/");
+  
   if (basename != NULL && basename[strlen (basename) -1] == '~')
     g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP, TRUE);
 
@@ -1477,20 +1482,32 @@ parse_attributes (GVfsBackendSftp *backend,
    * do better, since there is no way in this version of sftp to find out
    * the remote charset encoding
    */
-  if (basename != NULL &&
-      g_file_attribute_matcher_matches (matcher,
+  if (g_file_attribute_matcher_matches (matcher,
                                         G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME))
     {
-      char *display_name = g_filename_display_name (basename);
-      
-      if (strstr (display_name, "\357\277\275") != NULL)
+      if (basename != NULL)
         {
-          char *p = display_name;
-          display_name = g_strconcat (display_name, _(" (invalid encoding)"), NULL);
-          g_free (p);
+          char *display_name = g_filename_display_name (basename);
+          
+          if (strstr (display_name, "\357\277\275") != NULL)
+            {
+              char *p = display_name;
+              display_name = g_strconcat (display_name, _(" (invalid encoding)"), NULL);
+              g_free (p);
+            }
+          g_file_info_set_display_name (info, display_name);
+          g_free (display_name);
+        }
+      else
+        {
+          char *name;
+
+          
+          /* Translators: This is the name of the root of an sftp share, like "/ on <hostname>" */
+          name = g_strdup_printf (_("/ on %s"), G_VFS_BACKEND_SFTP (backend)->host);
+          g_file_info_set_display_name (info, name);
+          g_free (name);
         }
-      g_file_info_set_display_name (info, display_name);
-      g_free (display_name);
     }
   
   if (basename != NULL &&
@@ -1625,7 +1642,7 @@ open_for_read_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_HANDLE)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -1687,7 +1704,7 @@ read_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_DATA)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
   
@@ -1698,7 +1715,7 @@ read_reply (GVfsBackendSftp *backend,
                                 NULL, NULL, NULL))
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
   
@@ -1756,7 +1773,7 @@ seek_read_fstat_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_ATTRS)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -1846,7 +1863,7 @@ close_moved_tempfile (GVfsBackendSftp *backend,
     result_from_status (job, reply, -1, -1);
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 
   /* On failure, don't remove tempfile, since we removed the new original file */
   sftp_handle_free (handle);
@@ -1875,7 +1892,7 @@ close_deleted_file (GVfsBackendSftp *backend,
     res = error_from_status (job, reply, -1, -1, &error);
   else
     g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 _("Invalid reply recieved"));
+                 "%s", _("Invalid reply received"));
 
   if (res)
     {
@@ -1923,7 +1940,7 @@ close_moved_file (GVfsBackendSftp *backend,
     res = error_from_status (job, reply, -1, -1, &error);
   else
     g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 _("Invalid reply recieved"));
+                 "%s", _("Invalid reply received"));
 
   if (res)
     {
@@ -2002,7 +2019,7 @@ close_write_reply (GVfsBackendSftp *backend,
     res = error_from_status (job, reply, -1, -1, &error);
   else
     g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 _("Invalid reply recieved"));
+                 "%s", _("Invalid reply received"));
 
   if (res)
     {
@@ -2113,7 +2130,7 @@ close_read_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, -1, -1);
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
   sftp_handle_free (handle);
 }
 
@@ -2154,7 +2171,7 @@ create_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_HANDLE)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -2206,7 +2223,7 @@ append_to_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_HANDLE)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -2277,7 +2294,7 @@ replace_create_temp_reply (GVfsBackendSftp *backend,
       if (error_from_status (job, reply, G_IO_ERROR_EXISTS, -1, &error))
         /* Open should not return OK */
         g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                          _("Invalid reply recieved"));
+                          _("Invalid reply received"));
       else if (error->code == G_IO_ERROR_EXISTS)
         {
           /* It was *probably* the EXCL flag failing. I wish we had
@@ -2297,7 +2314,7 @@ replace_create_temp_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_HANDLE)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -2451,7 +2468,7 @@ replace_exclusive_reply (GVfsBackendSftp *backend,
       if (error_from_status (job, reply, G_IO_ERROR_EXISTS, -1, &error))
         /* Open should not return OK */
         g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                          _("Invalid reply recieved"));
+                          _("Invalid reply received"));
       else if (error->code == G_IO_ERROR_EXISTS)
         {
           /* It was *probably* the EXCL flag failing. I wish we had
@@ -2477,7 +2494,7 @@ replace_exclusive_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_HANDLE)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
   
@@ -2534,7 +2551,7 @@ write_reply (GVfsBackendSftp *backend,
     }
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static gboolean
@@ -2593,7 +2610,7 @@ seek_write_fstat_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_ATTRS)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -2875,7 +2892,7 @@ open_dir_reply (GVfsBackendSftp *backend,
   if (reply_type != SSH_FXP_HANDLE)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Invalid reply recieved"));
+                        _("Invalid reply received"));
       return;
     }
 
@@ -3004,11 +3021,13 @@ query_info_stat_reply (GVfsBackendSftp *backend,
     error_from_status (job, reply, -1, -1, &data->stat_error);
   else if (reply_type != SSH_FXP_ATTRS)
     g_set_error (&data->stat_error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 _("Invalid reply recieved"));
+                 "%s", _("Invalid reply received"));
   else
     {
       data->stat_info = g_file_info_new ();
-      basename = g_path_get_basename (G_VFS_JOB_QUERY_INFO (job)->filename);
+      basename = NULL;
+      if (strcmp (G_VFS_JOB_QUERY_INFO (job)->filename, "/") != 0)
+        basename = g_path_get_basename (G_VFS_JOB_QUERY_INFO (job)->filename);
       parse_attributes (backend, data->stat_info, basename,
                         reply, G_VFS_JOB_QUERY_INFO (job)->attribute_matcher);
       g_free (basename);
@@ -3035,11 +3054,13 @@ query_info_lstat_reply (GVfsBackendSftp *backend,
     error_from_status (job, reply, -1, -1, &data->lstat_error);
   else if (reply_type != SSH_FXP_ATTRS)
     g_set_error (&data->lstat_error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 _("Invalid reply recieved"));
+                 "%s", _("Invalid reply received"));
   else
     {
       data->lstat_info = g_file_info_new ();
-      basename = g_path_get_basename (G_VFS_JOB_QUERY_INFO (job)->filename);
+      basename = NULL;
+      if (strcmp (G_VFS_JOB_QUERY_INFO (job)->filename, "/") != 0)
+        basename = g_path_get_basename (G_VFS_JOB_QUERY_INFO (job)->filename);
       parse_attributes (backend, data->lstat_info, basename,
                         reply, G_VFS_JOB_QUERY_INFO (job)->attribute_matcher);
       g_free (basename);
@@ -3134,7 +3155,7 @@ move_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, G_IO_ERROR_NOT_SUPPORTED, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static gboolean
@@ -3172,10 +3193,10 @@ set_display_name_reply (GVfsBackendSftp *backend,
                         gpointer user_data)
 {
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1, -1); 
+    result_from_status (job, reply, -1, -1);
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static gboolean
@@ -3201,6 +3222,9 @@ try_set_display_name (GVfsBackend *backend,
   new_name = g_build_filename (dirname, basename, NULL);
   g_free (dirname);
   g_free (basename);
+
+  g_vfs_job_set_display_name_set_new_path (job,
+                                           new_name);
   
   command = new_command_stream (op_backend,
                                 SSH_FXP_RENAME,
@@ -3228,7 +3252,7 @@ make_symlink_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static gboolean
@@ -3264,7 +3288,7 @@ make_directory_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static gboolean
@@ -3298,7 +3322,7 @@ delete_remove_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static void
@@ -3313,7 +3337,7 @@ delete_rmdir_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, G_IO_ERROR_NOT_EMPTY, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
 }
 
 static void
@@ -3332,7 +3356,7 @@ delete_lstat_reply (GVfsBackendSftp *backend,
     result_from_status (job, reply, -1, -1);
   else if (reply_type != SSH_FXP_ATTRS)
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                      _("Invalid reply recieved"));
+                      _("Invalid reply received"));
   else
     {
       info = g_file_info_new ();
