@@ -219,7 +219,7 @@ unescape_pathname (const char *escaped_dir, int len)
             *d++ = '_';
           else
             {
-              c = *(p+1);
+              c = *p;
               if (c == '_')
                 {
                   p++;
@@ -592,8 +592,9 @@ g_vfs_backend_trash_init (GVfsBackendTrash *trash_backend)
 {
   GVfsBackend *backend = G_VFS_BACKEND (trash_backend);
   GMountSpec *mount_spec;
-  
-  g_vfs_backend_set_display_name (backend, _("Trashcan"));
+
+  /* translators: This is the name of the backend */
+  g_vfs_backend_set_display_name (backend, _("Trash"));
   g_vfs_backend_set_icon_name (backend, "user-trash");
   g_vfs_backend_set_user_visible (backend, FALSE);  
 
@@ -649,7 +650,7 @@ do_open_for_read (GVfsBackend *backend,
         {
           g_vfs_job_open_for_read_set_handle (job, stream);
           g_vfs_job_open_for_read_set_can_seek  (job,
-                                                 g_file_input_stream_can_seek (G_FILE_INPUT_STREAM (stream)));
+                                                 g_seekable_can_seek (G_SEEKABLE (stream)));
           g_vfs_job_succeeded (G_VFS_JOB (job));
         }
       else
@@ -710,13 +711,13 @@ do_seek_on_read (GVfsBackend *backend,
   stream = G_FILE_INPUT_STREAM (_handle);
 
   error = NULL;
-  if (g_file_input_stream_seek (stream,
-                                offset, type,
-                                G_VFS_JOB (job)->cancellable,
-                                &error))
+  if (g_seekable_seek (G_SEEKABLE (stream),
+                       offset, type,
+                       G_VFS_JOB (job)->cancellable,
+                       &error))
     {
       g_vfs_job_seek_read_set_offset (job,
-                                      g_file_input_stream_tell (stream));
+                                      g_seekable_tell (G_SEEKABLE (stream)));
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
   else
@@ -904,7 +905,7 @@ add_extra_trash_info (GFileInfo *file_info,
   char *orig_path, *orig_path_key, *orig_path_unescaped, *date;
   GKeyFile *keyfile;
   char *display_name;
-
+  char *desc;
 
   /* Override all writability */
   g_file_info_set_attribute_boolean (file_info,
@@ -916,6 +917,9 @@ add_extra_trash_info (GFileInfo *file_info,
   g_file_info_set_attribute_boolean (file_info,
                                      G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME,
                                      FALSE);
+  g_file_info_set_attribute_boolean (file_info,
+                                     G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
+                                     FALSE);
   
   /* But we can delete */
   g_file_info_set_attribute_boolean (file_info,
@@ -946,8 +950,18 @@ add_extra_trash_info (GFileInfo *file_info,
                   char *p = display_name;
                   display_name = g_strconcat (display_name, _(" (invalid encoding)"), NULL);
                   g_free (p);
+                  
+                  g_file_info_set_attribute_string (file_info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME, NULL);
                 }
+              else
+                g_file_info_set_attribute_string (file_info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME, display_name);
+                
               g_file_info_set_display_name (file_info, display_name);
+
+              desc = g_strdup_printf (_("%s (in trash)"), display_name);
+              g_file_info_set_attribute_string (file_info, G_FILE_ATTRIBUTE_STANDARD_DESCRIPTION, desc);
+              g_free (desc);
+              
               g_free (display_name);
               
               
@@ -960,7 +974,7 @@ add_extra_trash_info (GFileInfo *file_info,
 
               
               g_file_info_set_attribute_byte_string (file_info,
-                                                     "trash:orig_path",
+                                                     "trash::orig-path",
                                                      orig_path);
               g_free (orig_path);
               g_free (orig_path_unescaped);
@@ -973,7 +987,7 @@ add_extra_trash_info (GFileInfo *file_info,
       date = g_key_file_get_string (keyfile, "Trash Info", "DeletionDate", NULL);
       if (date && g_utf8_validate (date, -1, NULL))
         g_file_info_set_attribute_string (file_info,
-                                          "trash:deletion_date",
+                                          "trash::deletion-date",
                                           date);
       g_free (date);
     }
@@ -1161,7 +1175,9 @@ do_query_info (GVfsBackend *backend,
     {
       /* The trash:/// root */
       g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
-      g_file_info_set_display_name (info, _("Trashcan"));
+      g_file_info_set_name (info, "/");
+      /* Translators: this is the display name of the backend */
+      g_file_info_set_display_name (info, _("Trash"));
       g_file_info_set_content_type (info, "inode/directory");
 
       if (G_VFS_BACKEND_TRASH (backend)->top_files != NULL)
@@ -1187,6 +1203,9 @@ do_query_info (GVfsBackend *backend,
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE,
                                          FALSE);
+      g_file_info_set_attribute_boolean (info,
+                                         G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
+                                         FALSE);
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME,
                                          FALSE);
@@ -1200,6 +1219,7 @@ do_query_info (GVfsBackend *backend,
       char *path;
       GError *error; 
       char *info_dir;
+      char *basename;
      
       path = g_build_filename (trashdir, "files", trashfile, relative_path, NULL);
       file = g_file_new_for_path (path);
@@ -1217,6 +1237,10 @@ do_query_info (GVfsBackend *backend,
         {
           g_file_info_copy_into (local_info, info);
 
+          basename = g_path_get_basename (filename);
+          g_file_info_set_name (info, basename);
+          g_free (basename);
+      
           info_dir = g_build_filename (trashdir, "info", NULL);
           add_extra_trash_info (info,
                                 topdir,
