@@ -25,6 +25,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/errno.h>
+#include <errno.h>
 #include <sys/un.h>
 #include <unistd.h>
 #include <string.h>
@@ -42,7 +43,7 @@
 #include <gdbusutils.h>
 
 enum {
-  PROP_0,
+  PROP_0
 };
 
 typedef struct {
@@ -145,6 +146,7 @@ static void
 g_vfs_daemon_init (GVfsDaemon *daemon)
 {
   gint max_threads = 1; /* TODO: handle max threads */
+  DBusError error;
   
   daemon->lock = g_mutex_new ();
   daemon->session_bus = dbus_bus_get (DBUS_BUS_SESSION, NULL);
@@ -162,6 +164,20 @@ g_vfs_daemon_init (GVfsDaemon *daemon)
     g_hash_table_new_full (g_str_hash, g_str_equal,
 			   NULL, (GDestroyNotify)registered_path_free);
 
+  dbus_error_init (&error);
+  dbus_bus_add_match (daemon->session_bus,
+		      "type='signal',"		      
+		      "interface='org.freedesktop.DBus',"
+		      "member='NameOwnerChanged',"
+		      "arg0='"G_VFS_DBUS_DAEMON_NAME"'",
+		      &error);
+  
+  if (dbus_error_is_set (&error))
+    {
+      g_warning ("Failed to add dbus match: %s\n", error.message);
+      dbus_error_free (&error);
+    }
+  
   if (!dbus_connection_add_filter (daemon->session_bus,
 				   daemon_message_func, daemon, NULL))
     _g_dbus_oom ();
@@ -309,7 +325,7 @@ job_source_closed_callback (GVfsJobSource *job_source,
   g_mutex_lock (daemon->lock);
   
   daemon->job_sources = g_list_remove (daemon->job_sources,
-					     job_source);
+				       job_source);
   
   g_signal_handlers_disconnect_by_func (job_source,
 					(GCallback)job_source_new_job_callback,
@@ -326,6 +342,22 @@ job_source_closed_callback (GVfsJobSource *job_source,
   g_mutex_unlock (daemon->lock);
 }
 
+static void
+g_vfs_daemon_re_register_job_sources (GVfsDaemon *daemon)
+{
+  GList *l;
+  
+  g_mutex_lock (daemon->lock);
+
+  for (l = daemon->job_sources; l != NULL; l = l->next)
+    {
+      if (G_VFS_IS_BACKEND (l->data))
+	g_vfs_backend_register_mount (l->data, NULL, NULL);
+    }
+  
+  g_mutex_unlock (daemon->lock);
+}
+
 void
 g_vfs_daemon_add_job_source (GVfsDaemon *daemon,
 			     GVfsJobSource *job_source)
@@ -451,7 +483,7 @@ daemon_peer_connection_setup (GVfsDaemon *daemon,
   if (data->fd == -1)
     {
       /* The fd connection failed, abort the whole thing */
-      g_warning (_("Failed to accept client: %s"), _("accept of extra fd failed"));
+      g_warning ("Failed to accept client: %s", "accept of extra fd failed");
       dbus_connection_unref (dbus_conn);
       goto error_out;
     }
@@ -460,7 +492,7 @@ daemon_peer_connection_setup (GVfsDaemon *daemon,
   if (!dbus_connection_add_filter (dbus_conn, peer_to_peer_filter_func, daemon, NULL) ||
       !dbus_connection_add_filter (dbus_conn, daemon_message_func, daemon, NULL))
     {
-      g_warning (_("Failed to accept client: %s"), _("object registration failed"));
+      g_warning ("Failed to accept client: %s", "object registration failed");
       dbus_connection_unref (dbus_conn);
       close (data->fd);
       goto error_out;
@@ -698,7 +730,7 @@ accept_new_fd_client (GIOChannel  *channel,
   else if (data->fd == -1)
     {
       /* Didn't accept a dbus connection, and there is no need for one now */
-      g_warning (_("Failed to accept client: %s"), _("accept of extra fd failed"));
+      g_warning ("Failed to accept client: %s", "accept of extra fd failed");
       dbus_server_disconnect (data->server);
       dbus_server_unref (data->server);
       new_connection_data_free (data);
@@ -849,6 +881,8 @@ daemon_message_func (DBusConnection *conn,
   GVfsDaemon *daemon = data;
   RegisteredPath *registered_path;
   const char *path;
+  char *name;
+  char *old_owner, *new_owner;
 
   path = dbus_message_get_path (message);
   if (path == NULL)
@@ -856,7 +890,6 @@ daemon_message_func (DBusConnection *conn,
   
   if (dbus_message_is_signal (message, DBUS_INTERFACE_DBUS, "NameLost"))
     {
-      char *name;
       if (dbus_message_get_args (message, NULL,
 				 DBUS_TYPE_STRING, &name,
 				 DBUS_TYPE_INVALID) &&
@@ -867,7 +900,24 @@ daemon_message_func (DBusConnection *conn,
 	    exit (1);
 	}
     }
-  
+  else if (dbus_message_is_signal (message, DBUS_INTERFACE_DBUS, "NameOwnerChanged"))
+    {
+      if (dbus_message_get_args (message, NULL,
+				 DBUS_TYPE_STRING, &name,
+				 DBUS_TYPE_STRING, &old_owner,
+				 DBUS_TYPE_STRING, &new_owner,
+				 DBUS_TYPE_INVALID) &&
+	  strcmp (name, G_VFS_DBUS_DAEMON_NAME) == 0 &&
+	  *new_owner != 0 &&
+	  !daemon->main_daemon)
+	{
+	  /* There is a new owner. Register mounts with it */
+	  g_vfs_daemon_re_register_job_sources (daemon);
+	}
+      
+    }
+
+
   if (dbus_message_is_method_call (message,
 				   G_VFS_DBUS_DAEMON_INTERFACE,
 				   G_VFS_DBUS_OP_GET_CONNECTION))
