@@ -55,6 +55,7 @@ struct _GHalVolume {
    */
   GFile *foreign_mount_root;
   GMount *foreign_mount;
+  gboolean is_mountable;
 
   char *name;
   char *icon;
@@ -129,29 +130,29 @@ g_hal_volume_init (GHalVolume *hal_volume)
 }
 
 const static struct {
-        const char *disc_type;
-        const char *icon_name;
-        char *ui_name;
-        char *ui_name_blank;
+  const char *disc_type;
+  const char *icon_name;
+  char *ui_name;
+  char *ui_name_blank;
 } disc_data[] = {
-        {"cd_rom",        "media-optical-cd-rom", N_("CD-ROM Disc"), N_("Blank CD-ROM Disc")},
-        {"cd_r",          "media-optical-cd-r", N_("CD-R Disc"), N_("Blank CD-R Disc")},
-        {"cd_rw",         "media-optical-cd-rw", N_("CD-RW Disc"), N_("Blank CD-RW Disc")},
-        {"dvd_rom",       "media-optical-dvd-rom", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
-        {"dvd_ram",       "media-optical-dvd-ram", N_("DVD-RAM Disc"), N_("Blank DVD-RAM Disc")},
-        {"dvd_r",         "media-optical-dvd-r", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
-        {"dvd_rw",        "media-optical-dvd-rw", N_("DVD-RW Disc"), N_("Blank DVD-RW Disc")},
-        {"dvd_plus_r",    "media-optical-dvd-r-plus", N_("DVD+R Disc"), N_("Blank DVD+R Disc")},
-        {"dvd_plus_rw",   "media-optical-dvd-rw-plus",  N_("DVD+RW Disc"), N_("Blank DVD+RW Disc")},
-        {"dvd_plus_r_dl", "media-optical-dvd-dl-r-plus", N_("DVD+R DL Disc"), N_("Blank DVD+R DL Disc")},
-        {"bd_rom",        "media-optical-bd-rom", N_("Blu-Ray Disc"), N_("Blank Blu-Ray Disc")},
-        {"bd_r",          "media-optical-bd-r", N_("Blu-Ray R Disc"), N_("Blank Blu-Ray R Disc")},
-        {"bd_re",         "media-optical-bd-re", N_("Blu-Ray RW Disc"), N_("Blank Blu-Ray RW Disc")},
-        {"hddvd_rom",     "media-optical-hddvd-rom", N_("HD DVD Disc"), N_("Blank HD DVD Disc")},
-        {"hddvd_r",       "media-optical-hddvd-r", N_("HD DVD-R Disc"), N_("Blank HD DVD-R Disc")},
-        {"hddvd_rw",      "media-optical-hddvd-rw", N_("HD DVD-RW Disc"), N_("Blank HD DVD-RW Disc")},
-        {"mo",            "media-optical-mo", N_("MO Disc"), N_("Blank MO Disc")},
-        {NULL,            "media-optical", N_("Disc"), N_("Blank Disc")},
+  {"cd_rom",        "media-optical-cd-rom", N_("CD-ROM Disc"), N_("Blank CD-ROM Disc")},
+  {"cd_r",          "media-optical-cd-r", N_("CD-R Disc"), N_("Blank CD-R Disc")},
+  {"cd_rw",         "media-optical-cd-rw", N_("CD-RW Disc"), N_("Blank CD-RW Disc")},
+  {"dvd_rom",       "media-optical-dvd-rom", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
+  {"dvd_ram",       "media-optical-dvd-ram", N_("DVD-RAM Disc"), N_("Blank DVD-RAM Disc")},
+  {"dvd_r",         "media-optical-dvd-r", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
+  {"dvd_rw",        "media-optical-dvd-rw", N_("DVD-RW Disc"), N_("Blank DVD-RW Disc")},
+  {"dvd_plus_r",    "media-optical-dvd-r-plus", N_("DVD+R Disc"), N_("Blank DVD+R Disc")},
+  {"dvd_plus_rw",   "media-optical-dvd-rw-plus",  N_("DVD+RW Disc"), N_("Blank DVD+RW Disc")},
+  {"dvd_plus_r_dl", "media-optical-dvd-dl-r-plus", N_("DVD+R DL Disc"), N_("Blank DVD+R DL Disc")},
+  {"bd_rom",        "media-optical-bd-rom", N_("Blu-Ray Disc"), N_("Blank Blu-Ray Disc")},
+  {"bd_r",          "media-optical-bd-r", N_("Blu-Ray R Disc"), N_("Blank Blu-Ray R Disc")},
+  {"bd_re",         "media-optical-bd-re", N_("Blu-Ray RW Disc"), N_("Blank Blu-Ray RW Disc")},
+  {"hddvd_rom",     "media-optical-hddvd-rom", N_("HD DVD Disc"), N_("Blank HD DVD Disc")},
+  {"hddvd_r",       "media-optical-hddvd-r", N_("HD DVD-R Disc"), N_("Blank HD DVD-R Disc")},
+  {"hddvd_rw",      "media-optical-hddvd-rw", N_("HD DVD-RW Disc"), N_("Blank HD DVD-RW Disc")},
+  {"mo",            "media-optical-mo", N_("MO Disc"), N_("Blank MO Disc")},
+  {NULL,            "media-optical", N_("Disc"), N_("Blank Disc")},
 };
 
 static const char *
@@ -185,17 +186,17 @@ format_size_for_display (guint64 size)
   if (size < MEGABYTE_FACTOR)
     {
       displayed_size = (double) size / KILOBYTE_FACTOR;
-      str = g_strdup_printf (_("%.1f kB Media"), displayed_size);
+      str = g_strdup_printf (_("%.1f kB"), displayed_size);
     } 
   else if (size < GIGABYTE_FACTOR)
     {
       displayed_size = (double) size / MEGABYTE_FACTOR;
-      str = g_strdup_printf (_("%.1f MB Media"), displayed_size);
+      str = g_strdup_printf (_("%.1f MB"), displayed_size);
     } 
   else 
     {
       displayed_size = (double) size / GIGABYTE_FACTOR;
-      str = g_strdup_printf (_("%.1f GB Media"), displayed_size);
+      str = g_strdup_printf (_("%.1f GB"), displayed_size);
     }
   
   return str;
@@ -214,9 +215,12 @@ do_update_from_hal (GHalVolume *mv)
   gboolean volume_disc_has_data;
   const char *volume_disc_type;
   gboolean volume_disc_is_blank;
+  const char *volume_fsusage;
+  const char *volume_fstype;
   HalDevice *volume;
   HalDevice *drive;
   char *name;
+  char *size;
 
   volume = mv->device;
   drive = mv->drive_device;
@@ -231,27 +235,44 @@ do_update_from_hal (GHalVolume *mv)
   volume_disc_has_data = hal_device_get_property_bool (volume, "volume.disc.has_data");
   volume_disc_is_blank = hal_device_get_property_bool (volume, "volume.disc.is_blank");
   volume_disc_type = hal_device_get_property_string (volume, "volume.disc.type");
+  volume_fsusage = hal_device_get_property_string (volume, "volume.fsusage");
+  volume_fstype = hal_device_get_property_string (volume, "volume.fstype");
 
   if (volume_is_disc && volume_disc_has_audio && mv->foreign_mount_root != NULL)
-    {
-      name = g_strdup (_("Audio Disc"));
-    }
+    name = g_strdup (_("Audio Disc"));
   else
     {
-      if (volume_fs_label != NULL && strlen (volume_fs_label) > 0) {
-        name = g_strdup (volume_fs_label);
-      } else if (volume_is_disc) {
-        if (volume_disc_has_audio) {
-          if (volume_disc_has_data)
-            name = g_strdup (_("Mixed Audio/Data Disc"));
+      if (strcmp (volume_fsusage, "crypto") == 0 && strcmp (volume_fstype, "crypto_LUKS") == 0)
+        {
+          size = format_size_for_display (volume_size);
+          /* Translators: %s is the size of the volume (e.g. 512 MB) */
+          name = g_strdup_printf (_("%s Encrypted Data"), size);
+          g_free (size);
+        }
+      else
+        {
+          if (volume_fs_label != NULL && strlen (volume_fs_label) > 0)
+            name = g_strdup (volume_fs_label);
+          else if (volume_is_disc)
+            {
+              if (volume_disc_has_audio)
+                {
+                  if (volume_disc_has_data)
+                    name = g_strdup (_("Mixed Audio/Data Disc"));
+                  else
+                    name = g_strdup (_("Audio Disc"));
+                }
+              else
+                name = g_strdup (get_disc_name (volume_disc_type, volume_disc_is_blank));
+            }
           else
-            name = g_strdup (_("Audio Disc"));
-        } else {
-          name = g_strdup (get_disc_name (volume_disc_type, volume_disc_is_blank));
+            {
+              size = format_size_for_display (volume_size);
+              /* Translators: %s is the size of the volume (e.g. 512 MB) */
+              name = g_strdup_printf (_("%s Media"), size);
+              g_free (size);
+            }
         }
-      } else {
-        name = format_size_for_display (volume_size);
-      }
     }
 
   mv->name = name;
@@ -261,8 +282,50 @@ do_update_from_hal (GHalVolume *mv)
     mv->mount_path = g_strdup (hal_device_get_property_string (volume, "volume.mount_point"));
   else
     mv->mount_path = NULL;
+
+  g_object_set_data_full (G_OBJECT (mv), 
+                          "hal-storage-device-capabilities",
+                          g_strdupv (hal_device_get_property_strlist (mv->drive_device, "info.capabilities")),
+                          (GDestroyNotify) g_strfreev);
+
+  if (volume_disc_type != NULL && strlen (volume_disc_type) == 0)
+    volume_disc_type = NULL;
+  g_object_set_data_full (G_OBJECT (mv), 
+                          "hal-volume.disc.type",
+                          g_strdup (volume_disc_type),
+                          (GDestroyNotify) g_free);
 }
 
+#ifdef _WITH_GPHOTO2
+static void
+do_update_from_hal_for_camera (GHalVolume *v)
+{
+  const char *vendor;
+  const char *product;
+
+  vendor = hal_device_get_property_string (v->drive_device, "usb_device.vendor");
+  product = hal_device_get_property_string (v->drive_device, "usb_device.product");
+
+  if (vendor == NULL)
+    {
+      if (product != NULL)
+        v->name = g_strdup (product);
+      else
+        v->name = g_strdup (_("Camera"));
+    }
+  else
+    {
+      if (product != NULL)
+        v->name = g_strdup_printf ("%s %s", vendor, product);
+      else
+        v->name = g_strdup_printf (_("%s Camera"), vendor);
+    }
+
+  v->icon = g_strdup ("camera");
+  v->mount_path = NULL;
+}
+#endif
+
 static void
 update_from_hal (GHalVolume *mv, gboolean emit_changed)
 {
@@ -277,7 +340,14 @@ update_from_hal (GHalVolume *mv, gboolean emit_changed)
   g_free (mv->name);
   g_free (mv->icon);
   g_free (mv->mount_path);
+#ifdef _WITH_GPHOTO2
+  if (hal_device_has_capability (mv->device, "camera"))
+    do_update_from_hal_for_camera (mv);
+  else
+    do_update_from_hal (mv);
+#else
   do_update_from_hal (mv);
+#endif
 
   if (emit_changed)
     {
@@ -311,7 +381,7 @@ hal_changed (HalDevice *device, const char *key, gpointer user_data)
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (user_data);
   
-  //g_warning ("hal modifying %s (property %s changed)", hal_volume->device_path, key);
+  /*g_warning ("hal modifying %s (property %s changed)", hal_volume->device_path, key);*/
   update_from_hal (hal_volume, TRUE);
 }
 
@@ -329,18 +399,12 @@ compute_uuid (GHalVolume *volume)
   if (strlen (fs_uuid) == 0)
     {
       if (strlen (fs_label) == 0)
-        {
-          volume->uuid = NULL;
-        }
+        volume->uuid = NULL;
       else
-        {
-          volume->uuid = g_strdup (fs_label);
-        }
+        volume->uuid = g_strdup (fs_label);
     }
   else
-    {
-      volume->uuid = g_strdup (fs_uuid);
-    }
+    volume->uuid = g_strdup (fs_uuid);
 }
 
 GHalVolume *
@@ -348,28 +412,63 @@ g_hal_volume_new (GVolumeMonitor   *volume_monitor,
                   HalDevice        *device,
                   HalPool          *pool,
                   GFile            *foreign_mount_root,
+                  gboolean          is_mountable,
                   GHalDrive        *drive)
 {
   GHalVolume *volume;
   HalDevice *drive_device;
   const char *storage_udi;
+  const char *device_path;
       
-  storage_udi = hal_device_get_property_string (device, "block.storage_device");
-  if (storage_udi == NULL)
-    return NULL;
+  if (hal_device_has_capability (device, "block"))
+    {
+      storage_udi = hal_device_get_property_string (device, "block.storage_device");
+      if (storage_udi == NULL)
+        return NULL;
       
-  drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
-  if (drive_device == NULL)
-    return NULL;
-  
+      drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
+      if (drive_device == NULL)
+        return NULL;
+      
+      device_path = hal_device_get_property_string (device, "block.device");
+    }
+#ifdef _WITH_GPHOTO2
+  else if (hal_device_has_capability (device, "camera"))
+    {
+      /* OK, so we abuse storage_udi and drive_device for the USB main
+       * device that holds this interface... 
+       */
+      storage_udi = hal_device_get_property_string (device, "info.parent");
+      if (storage_udi == NULL)
+        return NULL;
+      
+      drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
+      if (drive_device == NULL)
+        return NULL;
+
+      /* TODO: other OS'es? Will address this with DK aka HAL 2.0 */
+      device_path = hal_device_get_property_string (drive_device, "linux.device_file");
+      if (strlen (device_path) == 0)
+        device_path = NULL;
+
+      if (foreign_mount_root == NULL)
+        return NULL;
+    }
+#endif
+  else
+    {
+      return NULL;
+    }
+
   volume = g_object_new (G_TYPE_HAL_VOLUME, NULL);
   volume->volume_monitor = volume_monitor;
   g_object_add_weak_pointer (G_OBJECT (volume_monitor), (gpointer) &(volume->volume_monitor));
   volume->mount_path = NULL;
-  volume->device_path = g_strdup (hal_device_get_property_string (device, "block.device"));
+  volume->device_path = g_strdup (device_path);
   volume->device = g_object_ref (device);
   volume->drive_device = g_object_ref (drive_device);
   volume->foreign_mount_root = foreign_mount_root != NULL ? g_object_ref (foreign_mount_root) : NULL;
+  volume->is_mountable = is_mountable;
   
   g_signal_connect_object (device, "hal_property_changed", (GCallback) hal_changed, volume, 0);
   g_signal_connect_object (drive_device, "hal_property_changed", (GCallback) hal_changed, volume, 0);
@@ -494,7 +593,8 @@ g_hal_volume_get_uuid (GVolume *volume)
 static gboolean
 g_hal_volume_can_mount (GVolume *volume)
 {
-  return TRUE;
+  GHalVolume *hal_volume = G_HAL_VOLUME (volume);
+  return hal_volume->is_mountable;
 }
 
 static gboolean
@@ -506,6 +606,15 @@ g_hal_volume_can_eject (GVolume *volume)
   return FALSE;
 }
 
+static gboolean
+g_hal_volume_should_automount (GVolume *volume)
+{
+  GHalVolume *hal_volume = G_HAL_VOLUME (volume);
+  /* TODO: For now, just never automount things that are not
+     local. Need to figure out a better approach later. */
+  return hal_volume->foreign_mount == NULL;
+}
+
 static GDrive *
 g_hal_volume_get_drive (GVolume *volume)
 {
@@ -538,6 +647,15 @@ g_hal_volume_has_mount_path (GHalVolume *volume,
   return FALSE;
 }
 
+gboolean
+g_hal_volume_has_device_path (GHalVolume *volume,
+                              const char  *device_path)
+{
+  if (volume->device_path != NULL)
+    return strcmp (volume->device_path, device_path) == 0;
+  return FALSE;
+}
+
 gboolean
 g_hal_volume_has_udi (GHalVolume  *volume,
                       const char  *udi)
@@ -664,18 +782,19 @@ spawn_do (GVolume             *volume,
                       NULL,         /* child_setup */
                       NULL,         /* user_data for child_setup */
                       &child_pid,
-                      &error)) {
-    GSimpleAsyncResult *simple;
-    simple = g_simple_async_result_new_from_error (data->object,
-                                                   data->callback,
-                                                   data->user_data,
-                                                   error);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
-    g_error_free (error);
-    g_free (data);
-    return;
-  }
+                      &error))
+    {
+      GSimpleAsyncResult *simple;
+      simple = g_simple_async_result_new_from_error (data->object,
+                                                     data->callback,
+                                                     data->user_data,
+                                                     error);
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
+      g_error_free (error);
+      g_free (data);
+      return;
+    }
   
   g_child_watch_add (child_pid, spawn_cb, data);
 }
@@ -699,6 +818,7 @@ mount_foreign_callback (GObject *source_object,
 
 static void
 g_hal_volume_mount (GVolume             *volume,
+                    GMountMountFlags     flags,
                     GMountOperation     *mount_operation,
                     GCancellable        *cancellable,
                     GAsyncReadyCallback  callback,
@@ -706,6 +826,11 @@ g_hal_volume_mount (GVolume             *volume,
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
 
+  /*g_warning ("hal_volume_mount (can_mount=%d foreign=%p device_path=%s)", 
+              g_hal_volume_can_mount (volume),
+              hal_volume->foreign_mount_root,
+              hal_volume->device_path);*/
+
   if (hal_volume->foreign_mount_root != NULL)
     {
       ForeignMountOp *data;
@@ -715,7 +840,8 @@ g_hal_volume_mount (GVolume             *volume,
       data->callback = callback;
       data->user_data = user_data;
 
-      g_file_mount_enclosing_volume (hal_volume->foreign_mount_root, 
+      g_file_mount_enclosing_volume (hal_volume->foreign_mount_root,
+                                     0,
                                      mount_operation, 
                                      cancellable, 
                                      mount_foreign_callback, 
@@ -723,8 +849,11 @@ g_hal_volume_mount (GVolume             *volume,
     }
   else
     {
-      char *argv[] = {"gnome-mount", "-b", "-d", NULL, NULL};
+      char *argv[] = {"gnome-mount", "-b", "-d", NULL, NULL, NULL};
       argv[3] = hal_volume->device_path;
+      /* ask for no dialogs if mount_operation is NULL */
+      if (mount_operation == NULL)
+        argv[4] = "-n";
       spawn_do (volume, cancellable, callback, user_data, argv);
     }
 }
@@ -737,13 +866,9 @@ g_hal_volume_mount_finish (GVolume       *volume,
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
 
   if (hal_volume->foreign_mount_root != NULL)
-    {
-      return g_file_mount_enclosing_volume_finish (hal_volume->foreign_mount_root, result, error);
-    }
-  else
-    {
-      return TRUE;
-    }
+    return g_file_mount_enclosing_volume_finish (hal_volume->foreign_mount_root, result, error);
+  
+  return TRUE;
 }
 
 typedef struct {
@@ -771,7 +896,7 @@ g_hal_volume_eject (GVolume              *volume,
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
 
-  g_warning ("hal_volume_eject");
+  /*g_warning ("hal_volume_eject");*/
 
   if (hal_volume->drive != NULL)
     {
@@ -792,12 +917,63 @@ g_hal_volume_eject_finish (GVolume        *volume,
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
 
   if (hal_volume->drive != NULL)
-    {
-      return g_drive_eject_finish (G_DRIVE (hal_volume->drive), result, error);
-    }
+    return g_drive_eject_finish (G_DRIVE (hal_volume->drive), result, error);
   return TRUE;
 }
 
+static char *
+g_hal_volume_get_identifier (GVolume              *volume,
+                             const char          *kind)
+{
+  GHalVolume *hal_volume = G_HAL_VOLUME (volume);
+
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_HAL_UDI) == 0)
+    return g_strdup (hal_device_get_udi (hal_volume->device));
+  
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE) == 0)
+    return g_strdup (hal_volume->device_path);
+  
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_LABEL) == 0)
+    return g_strdup (hal_device_get_property_string (hal_volume->device, "volume.label"));
+  
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UUID) == 0)
+    return g_strdup (hal_device_get_property_string (hal_volume->device, "volume.uuid"));
+  
+  return NULL;
+}
+
+static char **
+g_hal_volume_enumerate_identifiers (GVolume *volume)
+{
+  GHalVolume *hal_volume = G_HAL_VOLUME (volume);
+  GPtrArray *res;
+  const char *label, *uuid;
+
+  res = g_ptr_array_new ();
+
+  g_ptr_array_add (res,
+                   g_strdup (G_VOLUME_IDENTIFIER_KIND_HAL_UDI));
+
+  if (hal_volume->device_path && *hal_volume->device_path != 0)
+    g_ptr_array_add (res,
+                     g_strdup (G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE));
+    
+  label = hal_device_get_property_string (hal_volume->device, "volume.label");
+  uuid = hal_device_get_property_string (hal_volume->device, "volume.uuid");
+  
+  if (label && *label != 0)
+    g_ptr_array_add (res,
+                     g_strdup (G_VOLUME_IDENTIFIER_KIND_LABEL));
+
+  if (uuid && *uuid != 0)
+    g_ptr_array_add (res,
+                     g_strdup (G_VOLUME_IDENTIFIER_KIND_UUID));
+
+  /* Null-terminate */
+  g_ptr_array_add (res, NULL);
+  
+  return (char **)g_ptr_array_free (res, FALSE);
+}
 
 static void
 g_hal_volume_volume_iface_init (GVolumeIface *iface)
@@ -809,10 +985,13 @@ g_hal_volume_volume_iface_init (GVolumeIface *iface)
   iface->get_mount = g_hal_volume_get_mount;
   iface->can_mount = g_hal_volume_can_mount;
   iface->can_eject = g_hal_volume_can_eject;
+  iface->should_automount = g_hal_volume_should_automount;
   iface->mount_fn = g_hal_volume_mount;
   iface->mount_finish = g_hal_volume_mount_finish;
   iface->eject = g_hal_volume_eject;
   iface->eject_finish = g_hal_volume_eject_finish;
+  iface->get_identifier = g_hal_volume_get_identifier;
+  iface->enumerate_identifiers = g_hal_volume_enumerate_identifiers;
 }
 
 void 
